//===--- non_transitive_include-d1.h - test input file for iwyu -----------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

class NonTransitiveIncludeD1 {};


/**** IWYU_SUMMARY

(tests/cxx/non_transitive_include-d1.h has correct #includes/fwd-decls)

***** IWYU_SUMMARY */
