//===--- iwyu_version.h - include-what-you-use versioning -----------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef INCLUDE_WHAT_YOU_USE_IWYU_VERSION_H_
#define INCLUDE_WHAT_YOU_USE_IWYU_VERSION_H_

#define IWYU_VERSION_STRING "0.18"

#endif  // INCLUDE_WHAT_YOU_USE_IWYU_VERSION_H_
