//===--- cvr-derived.h - test input file for iwyu -------------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef CVR_DERIVED_H
#define CVR_DERIVED_H

#include "tests/cxx/cvr-base.h"

class Derived : public Base {};

#endif
