//===--- template_default_args_roundtrip-indirect.h - test input ----------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef INCLUDE_WHAT_YOU_USE_TESTS_CXX_TEMPLATE_DEFAULT_ARGS_ROUNDTRIP_INDIRECT_H_
#define INCLUDE_WHAT_YOU_USE_TESTS_CXX_TEMPLATE_DEFAULT_ARGS_ROUNDTRIP_INDIRECT_H_

#include "template_default_args_roundtrip-direct.h"

#endif  // INCLUDE_WHAT_YOU_USE_TESTS_CXX_TEMPLATE_DEFAULT_ARGS_ROUNDTRIP_INDIRECT_H_
