//===--- indirect.h - test input file for iwyu ----------------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef INCLUDE_WHAT_YOU_USE_TESTS_DRIVER_INDIRECT_H_
#define INCLUDE_WHAT_YOU_USE_TESTS_DRIVER_INDIRECT_H_

struct Indirect {
  int a;
};

#endif  // INCLUDE_WHAT_YOU_USE_TESTS_DRIVER_INDIRECT_H_
