//===--- enum_base-i1.h - test input file for iwyu ------------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef INCLUDE_WHAT_YOU_USE_TESTS_CXX_ENUM_BASE_I1_H_
#define INCLUDE_WHAT_YOU_USE_TESTS_CXX_ENUM_BASE_I1_H_

using int_t = int;

#endif  // INCLUDE_WHAT_YOU_USE_TESTS_CXX_ENUM_BASE_I1_H_
