//===--- fullinfo_for_templates-d1.h - test input file for iwyu -----------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef INCLUDE_WHAT_YOU_USE_TESTS_CXX_FULLINFO_FOR_TEMPLATES_D1_H_
#define INCLUDE_WHAT_YOU_USE_TESTS_CXX_FULLINFO_FOR_TEMPLATES_D1_H_

// Template definition.
template <typename T> class TemplateClass {};

#endif  // INCLUDE_WHAT_YOU_USE_TESTS_CXX_FULLINFO_FOR_TEMPLATES_D1_H_
