//===--- exitcode_good.c - test input file for iwyu -----------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

// Tests that IWYU exits with code 0 when no suggestions are made.

/**** IWYU_SUMMARY(0)

(tests/driver/exitcode_good.c has correct #includes/fwd-decls)

***** IWYU_SUMMARY */
