//===--- include_cycle-i1.h - test input file for iwyu --------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef INCLUDE_CYCLE_I1_H
#define INCLUDE_CYCLE_I1_H

#include "tests/cxx/internal/include_cycle-d1.h"

#endif  /* #ifndef INCLUDE_CYCLE_I1_H */
