//===--- badinc-i4.h - test input file for iwyu ---------------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef INCLUDE_WHAT_YOU_USE_TESTS_CXX_BADINC_I4_H_
#define INCLUDE_WHAT_YOU_USE_TESTS_CXX_BADINC_I4_H_

int I4_Function() { return 1; }

#endif // INCLUDE_WHAT_YOU_USE_TESTS_CXX_BADINC_I4_H_
