//===--- fwd_decl_class_template-i1.h - test input file for iwyu ----------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef INCLUDE_WHAT_YOU_USE_TESTS_CXX_FWD_DECL_CLASS_TEMPLATE_I1_H_
#define INCLUDE_WHAT_YOU_USE_TESTS_CXX_FWD_DECL_CLASS_TEMPLATE_I1_H_

template <typename T> class ClassTemplateI1 {};

#endif  // INCLUDE_WHAT_YOU_USE_TESTS_CXX_FWD_DECL_CLASS_TEMPLATE_I1_H_
