//===--- no_forced_alias_callability-d1.h - test input file for iwyu ------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

struct ReturnType;

struct Aliased {
  ReturnType doSomething();
};
