//===--- using_directive_decl.cc - test input file for iwyu ---------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
#include "using_directive_decl_a.h"
using namespace a;

/**** IWYU_SUMMARY

(tests/cxx/using_directive_decl.cc has correct #includes/fwd-decls)

***** IWYU_SUMMARY */
