//===--- macro_defined_by_includer-xmacro.h - test input file for iwyu ----===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

// File containing X-macro templates.

void f(TYPE const t) {}
