//===--- macro_defined_by_includer-prefix.h - test input file for iwyu ----===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#define ENABLE_DEBUG
#include "tests/cxx/macro_defined_by_includer-i4.h"
