//===--- redecls-d1.h - test input file for iwyu --------------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

// These are the redeclarable types, according to
//    http://clang.llvm.org/doxygen/classclang_1_1Redeclarable.html

typedef int MyTypedef;
void Fn();
extern float var;
