//===--- specialization_needs_decl-d1.h - test input file for iwyu --------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#include "tests/cxx/specialization_needs_decl-i1.h"

template <typename T> struct TplStruct { };

template <> struct TplStruct<float> { };
