//===--- direct_near.h - test input file for iwyu -------------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

// This file is the same as tests/direct.h, only intended to be used without
// provided header search path -I .

#ifndef INCLUDE_WHAT_YOU_USE_TESTS_DIRECT_NEAR_H_
#define INCLUDE_WHAT_YOU_USE_TESTS_DIRECT_NEAR_H_

#include "indirect.h"

#endif  // INCLUDE_WHAT_YOU_USE_TESTS_DIRECT_NEAR_H_
