//===--- no_fwd_decls-fwd.h - test input file for iwyu --------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef INCLUDE_WHAT_YOU_USE_TESTS_CXX_NO_FWD_DECLS_FWD_H_
#define INCLUDE_WHAT_YOU_USE_TESTS_CXX_NO_FWD_DECLS_FWD_H_

class Fwd;

#endif  // INCLUDE_WHAT_YOU_USE_TESTS_CXX_NO_FWD_DECLS-FWD_H_
