//===--- computed_include.h - test input file for iwyu --------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#define MACRO_INC "tests/cxx/indirect.h"

/**** IWYU_SUMMARY

(tests/cxx/computed_include.h has correct #includes/fwd-decls)

***** IWYU_SUMMARY */
