//===--- macro_defined_by_includer-d1.h - test input file for iwyu --------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

// Header file to provide extra level of inclusion indirection.

#include "tests/cxx/macro_defined_by_includer-i1.h"
